==> mvGenTL_Acquire-ARM64_gnu-2.40.2.tgz: mvIMPACT_Acquire-ARM64-2.40.2/kernelmodules/linux/mvBlueNAOS/mvdma.h <==
#ifndef __MVDMA_H__
#define __MVDMA_H__

/*
 *  mvPCIe Driver - DMA implementation
 *  Copyright (c) 2019-2020 by MATRIX VISION GmbH <info@matrix-vision.de>
 *  Author: grayh (based on previous work by mrakk and others)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#include "mvdma_ioctl.h"

#ifdef __KERNEL__

#include <linux/platform_device.h>
#include <linux/cdev.h>
#include <linux/io.h>
#include <linux/slab.h>
#include <linux/scatterlist.h>
#include <linux/pagemap.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/dma-mapping.h>
#include <linux/delay.h>

#include "mvpci.h"

/* Debug flags */
#define MVDMA_DEBUG_STD 0x001
#define MVDMA_DEBUG_MEM 0x002
#define MVDMA_DEBUG_RX 0x004
#define MVDMA_DEBUG_TX 0x008
#define MVDMA_DEBUG_DMA_DATA 0x010
#define MVDMA_DEBUG_MAPPING 0x020
#define MVDMA_DEBUG_DMA 0x040
#define MVDMA_DEBUG_DMA_CODE 0x080
#define MVDMA_DEBUG_QUEUE 0x080 /* ? */
#define MVDMA_DEBUG_TASKLET 0x100
#define MVDMA_DEBUG_IRQ 0x200

#define MAX_NUM_DMA_PER_DEVICE 1
#define MAX_NUM_DMA (MVPCI_MAX_DEVICES * MAX_NUM_DMA_PER_DEVICE)

#define dprintk_dma(lvl, fmt, args...)                                         \
    if ((pdma->debug & lvl) || !lvl)                                       \
    printk("%s-%d: " fmt, __FUNCTION__, pdma->id, ##args)

// versions without memory barriers
//#define mv_dma_read(addr) __raw_readl(pdma->mmio_base + addr)
//#define mv_dma_write(addr, val) __raw_writel(val, pdma->mmio_base + addr)

// versions with memory barriers
#define mv_dma_read(addr) readl(pdma->mmio_base + addr)
#define mv_dma_write(addr, val) writel(val, pdma->mmio_base + addr)

#define UPPER_WORD(n) ((u32)(n >> 32))
#define LOWER_WORD(n) ((u32)(n & 0x00000000ffffffffL))

#define MVDMA_CTRL_WORD             0x0800
#define MVDMA_CTRL_MAX_BURST_SIZE   0x0804
#define MVDMA_DESC_RING_ADR_UPPER   0x0808
#define MVDMA_DESC_RING_ADR_LOWER   0x080C
#define MVDMA_DESC_NUMBER           0x0810
#define MVDMA_DESC_NEW_WRITE        0x0814
#define MVDMA_FPGA_COUNTER          0x0830

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
typedef void mvdma_if_callback(void *data, struct timeval *tv, u32 img_size,
                   u32 img_id, u32 status);
#else
typedef void mvdma_if_callback(void *data, struct timespec64 *tv, u32 img_size,
                   u32 img_id, u32 status);
#endif

/* describes a single DMA buffer region for the DMA  -> maps to registers */
struct mvdma_hw_desc {
    u32 flags_count;    /* flags and command count */
    u32 cmd_addr_upper; /* upper part of DMA address of first associated page list (mvdma_page_list) */
    u32 cmd_addr_lower; /* lower part of DMA address of first associated page list (mvdma_page_list) */
} __attribute__((aligned(sizeof(u64)))) __attribute__ ((__packed__));


struct mvdma_queue {
    mvdma_if_callback *cb_user;

    void *userdata;
    struct scatterlist *sg;
    u32 valid_sgcnt;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
    struct timeval timestamp;
#else
    struct timespec64 timestamp;
#endif
    unsigned int img_size;
    atomic_t img_id;
};

/* list of DMA addresses to pages belonging to a buffer and, potentially, a
 * pointer to a subsequent list */
#define MAX_PAGE_LIST_SIZE_IN_KB 128
#define ADDR_BLOCK_MAX (((MAX_PAGE_LIST_SIZE_IN_KB * 1024) / sizeof(u64)) - 1)
struct __attribute__ ((__packed__)) mvdma_page_list {
    dma_addr_t dma_addr[ADDR_BLOCK_MAX];
    dma_addr_t next;
} __attribute__((aligned(sizeof(u64)))) __attribute__ ((__packed__));

/* forward declaration */
struct _TmvUI_userhandle;

/* an MV DMA device */
struct mvdma_device {
    struct mvpci_driver *mvpci_dev; /* parent device (mvpci_driver) */
    struct device *dev;        /* underlying Linux device */
    struct cdev cdev;        /* underlying character device */
    unsigned int id;     /* index - used to identify MV DMA device */
    void __iomem *mmio_base; /* where to find the registers */
    struct _TmvUI_userhandle *uh;
    unsigned int handle_used;

    spinlock_t lock;

    volatile u32 dma_queue_first;
    volatile u32 dma_queue_entries;
    volatile u32 fpga_leading_by;
    unsigned int num_dma_descriptors; /* how many DMA descriptors could be reserved  (<= MVDMA_QUEUE_CNT) */
    DECLARE_BITMAP(dma_queue_deliver, MVDMA_QUEUE_CNT);
    DECLARE_BITMAP(dma_queue_invalid, MVDMA_QUEUE_CNT);

    struct mvdma_queue *dma_queue[MVDMA_QUEUE_CNT];     /* queue infos */
    struct mvdma_hw_desc *dma_desc[MVDMA_QUEUE_CNT];    /* descriptors for DMA */
    struct mvdma_page_list *page_list[MVDMA_QUEUE_CNT]; /* page lists used to describe buffers */

    dma_addr_t dma_handle[MVDMA_QUEUE_CNT];         /* DMA addresses of dma_desc */
    dma_addr_t dma_addr_page_list[MVDMA_QUEUE_CNT]; /* DMA addresses of page_lists */

    struct tasklet_struct irq_tasklet;
    u32 debug; /* keep track of debug level */
};

#define _get_free_queue(d)                                                     \
    ((d->dma_queue_first + d->dma_queue_entries) % MVDMA_QUEUE_CNT)

enum mv_dma_state { DMA_IDLE = 0, DMA_DONE, DMA_RUNNING, DMA_MAX };

typedef struct _TmvUI_umem {
    void *uh;
    struct page **pages;
    int page_cnt;
    size_t len;
    size_t offset;
    unsigned long uva;
    unsigned long flags;
    struct timeval tv;
    unsigned int img_size;
    unsigned int img_id;
    int nr;
    int dma_state;
    wait_queue_head_t wq;
    struct delayed_work work;
    struct scatterlist *sl;
    struct mutex lock;
} TmvUI_umem;

typedef struct _TmvUI_userhandle {
    struct mvdma_device *pdma;
    struct mutex umem_lock;
    TmvUI_umem umem[MVDMA_QUEUE_CNT];
    int last_umem;
    unsigned long image_nr;
} TmvUI_userhandle;

static inline u32 CHECK_IO_SIZE(u32 real, u32 expected)
{
    if (real < expected) {
        printk(KERN_ERR
               "mvpci : ioctl size mismatch %d != %d"
               " -> please use latest drivers !\n",
               real, expected);
        return real;
    }
    return expected;
}

extern int mvdma_probe(struct mvpci_driver *mvpci_dev);
extern int mvdma_remove(struct mvpci_driver *mvpci_dev);
extern long mvdma_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);
extern int mvdma_request(void *handle, struct scatterlist *sg, u32 sgcnt,
             mvdma_if_callback *cb, void *data);
extern ssize_t mvdma_dma_to_user(TmvUI_umem *um);
extern irqreturn_t mvdma_irq(int irq, void *context);
extern int mvdma_reset(void *handle);

#endif //__KERNEL__
#endif //__MVDMA_H__

==> mvGenTL_Acquire-ARM64_gnu-2.41.0.tgz: mvIMPACT_Acquire-ARM64-2.41.0/kernelmodules/linux/mvBlueNAOS/mvdma.h <==
#ifndef __MVDMA_H__
#define __MVDMA_H__

/*
 *  mvPCIe Driver - DMA implementation
 *  Copyright (c) 2019-2021 by MATRIX VISION GmbH <info@matrix-vision.de>
 *  Author: grayh (based on previous work by mrakk and others)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#include "mvdma_ioctl.h"

#ifdef __KERNEL__

#include <linux/platform_device.h>
#include <linux/cdev.h>
#include <linux/io.h>
#include <linux/slab.h>
#include <linux/scatterlist.h>
#include <linux/pagemap.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/dma-mapping.h>
#include <linux/delay.h>

#include "mvpci.h"

/* Debug flags */
#define MVDMA_DEBUG_STD     0x0001
#define MVDMA_DEBUG_MEM     0x0002
#define MVDMA_DEBUG_MAPPING 0x0020
#define MVDMA_DEBUG_QUEUE   0x0080
#define MVDMA_DEBUG_TASKLET 0x0100
#define MVDMA_DEBUG_IRQ     0x0200

#define MAX_NUM_DMA_PER_DEVICE 1
#define MAX_NUM_DMA (MVPCI_MAX_DEVICES * MAX_NUM_DMA_PER_DEVICE)

#define dprintk_dma(lvl, fmt, args...)                                         \
    if ((pdma->debug & lvl) || !lvl)                                       \
    printk("%s-%d: " fmt, __FUNCTION__, pdma->id, ##args)

// versions with memory barriers
#define mv_dma_read(addr) readl(pdma->mmio_base + addr)
#define mv_dma_write(addr, val) writel(val, pdma->mmio_base + addr)

#define UPPER_WORD(n) ((u32)(n >> 32))
#define LOWER_WORD(n) ((u32)(n & 0x00000000ffffffffL))

#define MVDMA_CTRL_WORD             0x0800
#define MVDMA_CTRL_MAX_BURST_SIZE   0x0804
#define MVDMA_DESC_RING_ADR_UPPER   0x0808
#define MVDMA_DESC_RING_ADR_LOWER   0x080C
#define MVDMA_DESC_NUMBER           0x0810
#define MVDMA_DESC_NEW_WRITE        0x0814
#define MVDMA_FPGA_COUNTER          0x0830

typedef void mvdma_if_callback(void *data, u32 img_size, u32 img_id);

/* describes a single DMA buffer region for the DMA  -> maps to registers */
struct mvdma_hw_desc {
    u32 flags_count;    /* flags and command count */
    u32 cmd_addr_upper; /* upper part of DMA address of first associated page list (mvdma_page_list) */
    u32 cmd_addr_lower; /* lower part of DMA address of first associated page list (mvdma_page_list) */
} __attribute__((aligned(sizeof(u64)))) __attribute__ ((__packed__));


struct mvdma_queue {
    mvdma_if_callback *cb_user;

    void *userdata;
    struct scatterlist *sg;
    u32 sgcnt;
    u32 valid_sgcnt;
    unsigned int img_size;
    atomic_t img_id;
};

/* list of DMA addresses to pages belonging to a buffer and, potentially, a
 * pointer to a subsequent list */
#define MAX_PAGE_LIST_SIZE_IN_KB 128
#define ADDR_BLOCK_MAX (((MAX_PAGE_LIST_SIZE_IN_KB * 1024) / sizeof(u64)) - 1)
struct __attribute__ ((__packed__)) mvdma_page_list {
    dma_addr_t dma_addr[ADDR_BLOCK_MAX];
    dma_addr_t next;
} __attribute__((aligned(sizeof(u64)))) __attribute__ ((__packed__));

/* forward declaration */
struct _TmvUI_userhandle;

/* an MV DMA device */
struct mvdma_device {
    struct mvpci_driver *mvpci_dev; /* parent device (mvpci_driver) */
    struct device *dev;        /* underlying Linux device */
    struct cdev cdev;        /* underlying character device */
    unsigned int id;     /* index - used to identify MV DMA device */
    void __iomem *mmio_base; /* where to find the registers */
    struct _TmvUI_userhandle *uh;
    unsigned int handle_used;

    spinlock_t lock;

    volatile u32 dma_queue_first;
    volatile u32 dma_queue_entries;
    volatile u32 fpga_leading_by;
    unsigned int num_dma_descriptors; /* how many DMA descriptors could be reserved  (<= MVDMA_QUEUE_CNT) */
    DECLARE_BITMAP(dma_queue_deliver, MVDMA_QUEUE_CNT);
    DECLARE_BITMAP(dma_queue_invalid, MVDMA_QUEUE_CNT);

    struct mvdma_queue *dma_queue[MVDMA_QUEUE_CNT];     /* queue infos */
    struct mvdma_hw_desc *dma_desc[MVDMA_QUEUE_CNT];    /* descriptors for DMA */
    struct mvdma_page_list *page_list[MVDMA_QUEUE_CNT]; /* page lists used to describe buffers */

    dma_addr_t dma_handle[MVDMA_QUEUE_CNT];         /* DMA addresses of dma_desc */
    dma_addr_t dma_addr_page_list[MVDMA_QUEUE_CNT]; /* DMA addresses of page_lists */

    struct tasklet_struct irq_tasklet;
    u32 debug; /* keep track of debug level */
};

#define _get_free_queue(d)                                                     \
    ((d->dma_queue_first + d->dma_queue_entries) % MVDMA_QUEUE_CNT)

enum mv_dma_state { DMA_IDLE = 0, DMA_DONE, DMA_RUNNING, DMA_MAX };

typedef struct _TmvUI_umem {
    void *uh;
    struct page **pages;
    int page_cnt;
    size_t len;
    size_t offset;
    unsigned long uva;
    unsigned long flags;
    struct timespec64 ts;
    unsigned int img_size;
    unsigned int img_id;
    u32 nr;
    enum mv_dma_state dma_state;
    wait_queue_head_t wq;
    struct delayed_work work;
    struct scatterlist *sl;
    struct mutex lock;
} TmvUI_umem;

typedef struct _TmvUI_userhandle {
    struct mvdma_device *pdma;
    struct mutex umem_lock;
    TmvUI_umem umem[MVDMA_QUEUE_CNT];
    int last_umem;
    unsigned long image_nr;
} TmvUI_userhandle;

static inline u32 CHECK_IO_SIZE(u32 real, u32 expected)
{
    if (real < expected) {
        printk(KERN_ERR
               "mvpci : ioctl size mismatch %d != %d"
               " -> please use latest drivers !\n",
               real, expected);
        return real;
    }
    return expected;
}

extern int mvdma_probe(struct mvpci_driver *mvpci_dev);
extern int mvdma_remove(struct mvpci_driver *mvpci_dev);
extern long mvdma_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);
extern int mvdma_request(void *handle, struct scatterlist *sg, u32 sgcnt,
             mvdma_if_callback *cb, void *data);
extern irqreturn_t mvdma_irq(int irq, void *context);
extern int mvdma_reset(void *handle);

#endif //__KERNEL__
#endif //__MVDMA_H__
